@@ -21,10 +21,18 @@
 #define __USER32_H__
 
 
-#include "export.h"
+#include "pe_image.h"
+
+
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 
 SymbolInfo *user32_get_export_symbols(void);
 
 
+#ifdef __cplusplus
+}
+#endif
 #endif
