@@ -128,8 +128,16 @@ fileio_test (const gchar *file,
 /*  ja:ファイルのフルパスを取得する
     file,ファイル名
      RET,フルパス                                                           */
+#define fileio_get_full_path(file) fileio_get_absolute_path(file,NULL)
+
+
+/*  ja:ファイルの絶対パスを取得する
+    file,ファイル名
+    path,パス名
+     RET,フルパス                                                           */
 gchar *
-fileio_get_full_path (const gchar *file);
+fileio_get_absolute_path (const gchar *file,
+                          const gchar *path);
 
 
 /*  ja:ファイルの相対パスを取得する
