@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,10 +24,16 @@
 #include "sigfile.h"
 #include "size.h"
 #include "thread.h"
-#include "avicore/avifilesel.h"
+#if GTK_CHECK_VERSION(2,4,0)
+# include "avicore/avifileextra.h"
+#else /* not GTK_CHECK_VERSION(2,4,0) */
+# include "avicore/avifilesel.h"
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
 #include "misc/fileio.h"
 #include "misc/misc.h"
 #include "misc/misc_conf.h"
+#include "misc/misc_history.h"
+#include "misc/misc_mdi.h"
 #ifdef G_OS_WIN32
 # include <gdk/gdkwin32.h>
 #endif /* G_OS_WIN32 */
@@ -61,6 +67,7 @@ struct _CommandOpenOk
 };
 
 
+#if ! GTK_CHECK_VERSION(2,4,0)
 static void
 command_open_ok (GtkWidget     *widget,
                  CommandOpenOk *cmd_ok)
@@ -77,6 +84,7 @@ command_open_ok (GtkWidget     *widget,
 
   gtk_widget_destroy (cmd_ok->dialog);
 }
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
 
 
 void
@@ -84,10 +92,34 @@ command_open (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
+  gchar *path;
   CommandOpenOk cmd_ok;
   FileOpen *file_open;
+#if GTK_CHECK_VERSION(2,4,0)
+  GtkWidget *check;
+#endif /* GTK_CHECK_VERSION(2,4,0) */
 
   cmd_ok.file = NULL;
+#if GTK_CHECK_VERSION(2,4,0)
+  cmd_ok.dialog = gtk_file_chooser_dialog_new (_("Open"),
+                            GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_OPEN,
+                            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            NULL);
+  if (open_path)
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (cmd_ok.dialog),
+                                                                    open_path);
+  check = gtk_check_button_new_with_label (_("Continual"));
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), file_merge);
+  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (cmd_ok.dialog), check);
+  if (gtk_dialog_run (GTK_DIALOG (cmd_ok.dialog)) == GTK_RESPONSE_ACCEPT)
+    {
+      cmd_ok.file = gtk_file_chooser_get_filename
+                                            (GTK_FILE_CHOOSER (cmd_ok.dialog));
+      cmd_ok.merge = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check));
+    }
+  gtk_widget_destroy (cmd_ok.dialog);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   cmd_ok.dialog = avi_file_selection_new (_("Open"));
   g_signal_connect (G_OBJECT (cmd_ok.dialog), "destroy", gtk_main_quit, NULL);
   g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (cmd_ok.dialog)->ok_button),
@@ -97,8 +129,12 @@ command_open (gpointer   callback_data,
                                     "clicked", G_CALLBACK (gtk_widget_destroy),
                                                     G_OBJECT (cmd_ok.dialog));
   if (open_path)
-    gtk_file_selection_set_filename
-                            (GTK_FILE_SELECTION (cmd_ok.dialog), open_path);
+    {
+      path = g_strconcat (open_path, G_SEARCHPATH_SEPARATOR_S, NULL);
+      gtk_file_selection_set_filename
+                                    (GTK_FILE_SELECTION (cmd_ok.dialog), path);
+      g_free (path);
+    }
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (cmd_ok.dialog));
   avi_file_selection_set_continual (AVI_FILE_SELECTION (cmd_ok.dialog),
                                     file_merge);
@@ -106,11 +142,13 @@ command_open (gpointer   callback_data,
   gtk_widget_show (cmd_ok.dialog);
   gtk_grab_add (cmd_ok.dialog);
   gtk_main ();
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   if (cmd_ok.file)
     {
       g_free (open_path);
-      open_path = fileio_get_full_path (cmd_ok.file);
-      *(gchar *)g_basename (open_path) = '\0';
+      path = fileio_get_full_path (cmd_ok.file);
+      open_path = g_path_get_dirname (path);
+      g_free (path);
       file_open = g_malloc (sizeof (FileOpen));
       file_open->file = cmd_ok.file;
       file_open->merge = file_merge = cmd_ok.merge;
@@ -129,16 +167,10 @@ command_close (gpointer   callback_data,
                GtkWidget *widget)
 {
   gint page;
-  VmaidWindow *vmaid;
 
-  page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
-  if (page >= 0)
-    {
-      vmaid = g_object_get_data (G_OBJECT (gtk_notebook_get_nth_page
-                                (GTK_NOTEBOOK (notebook), page)), "user_data");
-      if (prompt_close (vmaid))
-        gtk_notebook_remove_page(GTK_NOTEBOOK(notebook),page);
-    }
+  if ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi))) >= 0
+    && prompt_close ((VmaidWindow *)misc_mdi_get_data (MISC_MDI (mdi), page)))
+    gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page);
 }
 
 
@@ -147,16 +179,17 @@ command_save (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
+  gchar *path;
   gint i = 0;
   AviEdit *avi_edit[3];
   CommandOpenOk cmd_ok;
   FileSave *file_save;
   VmaidWindow *vmaid;
+#if GTK_CHECK_VERSION(2,4,0)
+  GtkWidget *extra;
+#endif /* GTK_CHECK_VERSION(2,4,0) */
 
-  vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  vmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   if (vmaid->avi_edit[0])
     avi_edit[i++] = vmaid->avi_edit[0];
   if (vmaid->avi_edit[1])
@@ -164,6 +197,26 @@ command_save (gpointer   callback_data,
   avi_edit[i] = NULL;
   cmd_ok.file = NULL;
   cmd_ok.avi_save = NULL;
+#if GTK_CHECK_VERSION(2,4,0)
+  cmd_ok.dialog = gtk_file_chooser_dialog_new (_("Save As"),
+                            GTK_WINDOW (window), GTK_FILE_CHOOSER_ACTION_SAVE,
+                            GTK_STOCK_SAVE_AS, GTK_RESPONSE_ACCEPT,
+                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+                            NULL);
+  if (save_path)
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (cmd_ok.dialog),
+                                                                    save_path);
+  extra = avi_file_extra_new ();
+  avi_file_extra_set_handle (AVI_FILE_EXTRA (extra), avi_edit);
+  gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (cmd_ok.dialog), extra);
+  if (gtk_dialog_run (GTK_DIALOG (cmd_ok.dialog)) == GTK_RESPONSE_ACCEPT)
+    {
+      cmd_ok.file = gtk_file_chooser_get_filename
+                                            (GTK_FILE_CHOOSER (cmd_ok.dialog));
+      cmd_ok.avi_save = avi_file_extra_get_option (AVI_FILE_EXTRA (extra));
+    }
+  gtk_widget_destroy (cmd_ok.dialog);
+#else /* not GTK_CHECK_VERSION(2,4,0) */
   cmd_ok.dialog = avi_file_selection_new (_("Save As"));
   g_signal_connect (G_OBJECT (cmd_ok.dialog), "destroy", gtk_main_quit, NULL);
   g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (cmd_ok.dialog)->ok_button),
@@ -173,26 +226,33 @@ command_save (gpointer   callback_data,
                                     "clicked", G_CALLBACK (gtk_widget_destroy),
                                                     G_OBJECT (cmd_ok.dialog));
   if (save_path)
-    gtk_file_selection_set_filename
-                            (GTK_FILE_SELECTION (cmd_ok.dialog), save_path);
+    {
+      path = g_strconcat (save_path, G_SEARCHPATH_SEPARATOR_S, NULL);
+      gtk_file_selection_set_filename
+                                    (GTK_FILE_SELECTION (cmd_ok.dialog), path);
+      g_free (path);
+    }
   gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (cmd_ok.dialog));
   avi_file_selection_set_handle (AVI_FILE_SELECTION (cmd_ok.dialog), avi_edit);
   gtk_widget_show (cmd_ok.dialog);
   gtk_grab_add (cmd_ok.dialog);
   gtk_main ();
+#endif /* not GTK_CHECK_VERSION(2,4,0) */
   if (cmd_ok.file && cmd_ok.avi_save)
     {
       g_free (save_path);
-      save_path = fileio_get_full_path (cmd_ok.file);
-      *(gchar *)g_basename (save_path) = '\0';
+      path = fileio_get_full_path (cmd_ok.file);
+      save_path = g_path_get_dirname (path);
+      g_free (path);
       if (avi_save_is_all (avi_edit, cmd_ok.avi_save))
         {
           /* ja:すべて保存されるとき */
-          vmaid_change_edit (vmaid, FALSE);
+          misc_mdi_set_edited (MISC_MDI (mdi),
+                misc_mdi_get_page_from_data (MISC_MDI (mdi), vmaid), FALSE);
           delete_list (&vmaid->undo);
           delete_list (&vmaid->redo);
           set_menu_bar (vmaid);
-          file_set_history (cmd_ok.file);
+          misc_history_add_file (MISC_HISTORY (history), cmd_ok.file);
         }
       file_save = g_malloc (sizeof (FileSave));
       file_save->file = cmd_ok.file;
@@ -209,10 +269,8 @@ command_save (gpointer   callback_data,
     }
   else
     {
-      if (cmd_ok.file)
-        g_free (cmd_ok.file);
-      if (cmd_ok.avi_save)
-        avi_save_free (cmd_ok.avi_save);
+      g_free (cmd_ok.file);
+      avi_save_free (cmd_ok.avi_save);
     }
 }
 
@@ -225,10 +283,7 @@ command_property (gpointer   callback_data,
   VmaidWindow *vmaid;
   PropertyDialog prop;
 
-  vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  vmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   g_memset (&prop, 0, sizeof (PropertyDialog));
   prop.rate = vmaid->rate;
   prop.scale = vmaid->scale;
@@ -323,13 +378,8 @@ command_undo (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
-  VmaidWindow *vmaid;
-
-  vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-  history_operation (vmaid, FALSE);
+  history_operation ((VmaidWindow *)misc_mdi_get_data (MISC_MDI (mdi), -1),
+                                                                        FALSE);
 }
 
 
@@ -338,13 +388,8 @@ command_redo (gpointer   callback_data,
               guint      callback_action,
               GtkWidget *widget)
 {
-  VmaidWindow *vmaid;
-
-  vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-  history_operation (vmaid, TRUE);
+  history_operation ((VmaidWindow *)misc_mdi_get_data (MISC_MDI (mdi), -1),
+                                                                        TRUE);
 }
 
 
@@ -372,10 +417,7 @@ command_copy (gpointer   callback_data,
                                                             GDK_CURRENT_TIME))
     {
 #endif /* not G_OS_WIN32 */
-      vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+      vmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
       /* ja:ビデオ */
       if (MIN (vmaid->cursor.stream, vmaid->select.stream) == 0)
         {
@@ -509,19 +551,15 @@ command_delete (gpointer   callback_data,
   VmaidHistory *d;
   VmaidWindow *vmaid;
 
-  vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
-
+  vmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   /* ja:UNDOのための処理 */
   d = g_malloc (sizeof (VmaidHistory));
   for (i = 0; i < 2; i++)
     {
       d->avi_edit[i] = NULL;
       if (vmaid->avi_edit[i]
-                        && (d->avi_edit[i] = avi_clone (vmaid->avi_edit[i])))
-        result=FALSE;
+                        && !(d->avi_edit[i] = avi_clone (vmaid->avi_edit[i])))
+        result = FALSE;
     }
   if (!result)
     {
@@ -624,7 +662,8 @@ command_delete (gpointer   callback_data,
   misc_set_scroll_bar (vmaid->hscroll,
                                     G_CALLBACK (signal_value_changed_hscroll),
                                                 vmaid, 0, max, sx, vmaid->top);
-  vmaid_change_edit (vmaid, TRUE);
+  misc_mdi_set_edited (MISC_MDI (mdi),
+                    misc_mdi_get_page_from_data (MISC_MDI (mdi), vmaid), TRUE);
 }
 
 
@@ -638,10 +677,7 @@ command_jump (gpointer   callback_data,
   VmaidCursor cursor, select;
   VmaidWindow *vmaid;
 
-  vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  vmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   cursor = vmaid->cursor;
   select = vmaid->select;
   top = vmaid->top;
@@ -687,10 +723,7 @@ command_all (gpointer   callback_data,
   VmaidCursor cursor, select;
   VmaidWindow *vmaid;
 
-  vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  vmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   if (vmaid->avi_edit[0] || vmaid->avi_edit[1])
     {
       cursor = vmaid->cursor;
@@ -727,10 +760,7 @@ command_size (gpointer   callback_data,
   SizeDialog size;
   VmaidWindow *vmaid;
 
-  vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+  vmaid = misc_mdi_get_data (MISC_MDI (mdi), -1);
   size.view = default_view;
   size.ratio = default_ratio;
   size.width = vmaid->width;
@@ -764,26 +794,15 @@ static void
 command_conf_ok (GtkWidget *widget,
                  GtkWidget *dialog)
 {
-  gint i;
-  GList *glist;
-  GtkWidget *menu_shell;
-
-  history = misc_conf_get_history (MISC_CONF (dialog));
+  misc_history_set_num (MISC_HISTORY (history),
+                                misc_conf_get_history (MISC_CONF (dialog)));
   newfile = misc_conf_get_newfile (MISC_CONF (dialog));
   second = misc_conf_get_second (MISC_CONF (dialog));
   misc_conf_get_size (MISC_CONF (dialog), &def_width, &def_height);
   n_pos = misc_conf_get_tab (MISC_CONF (dialog));
   gtk_widget_destroy (dialog);
-  /* ja:履歴の整理 */
-  menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/File");
-  glist = gtk_container_children (GTK_CONTAINER (menu_shell));
-  for (i = g_list_length (glist) - 3;
-                            i > history- (history == 0) + MENUFILE - 2; i--)
-    gtk_container_remove (GTK_CONTAINER (menu_shell),
-                                                g_list_nth_data (glist, i));
-  g_list_free (glist);
   /* ja:ノートの位置 */
-  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), n_pos);
+  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (mdi), n_pos);
 }
 
 
@@ -801,7 +820,8 @@ command_conf (gpointer   callback_data,
                             "clicked", G_CALLBACK (command_conf_ok), dialog);
   g_signal_connect_swapped (G_OBJECT (MISC_CONF (dialog)->cancel_button),
             "clicked", G_CALLBACK (gtk_widget_destroy), GTK_OBJECT (dialog));
-  misc_conf_set_history (MISC_CONF(dialog), history);
+  misc_conf_set_history (MISC_CONF(dialog),
+                                misc_history_get_num (MISC_HISTORY (history)));
   misc_conf_set_newfile (MISC_CONF(dialog), newfile);
   misc_conf_set_second (MISC_CONF(dialog), second);
   misc_conf_set_size (MISC_CONF(dialog), def_width, def_height);
