@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,11 +24,13 @@
 #include "thread.h"
 #include "misc/argument.h"
 #include "misc/misc.h"
+#include "misc/misc_history.h"
+#include "misc/misc_mdi.h"
 
 
 /******************************************************************************
 *                                                                             *
-* ja:シグナル/イベント関数群(テキスト)                                         *
+* ja:シグナル/イベント関数群(テキスト)                                        *
 *                                                                             *
 ******************************************************************************/
 void
@@ -37,8 +39,6 @@ signal_style_set (GtkWidget *widget,
                   gpointer   user_data)
 {
   gint i, ascent, height;
-  GList *glist;
-  VmaidWindow *vmaid;
 
   /* ja:システム色 */
   style = gtk_widget_get_style (widget);
@@ -58,20 +58,19 @@ signal_style_set (GtkWidget *widget,
         system_font_ascent = ascent;
       height = gdk_char_height (system_font, (gchar)i);
       if (system_font_height < height)
-        system_font_height=height;
+        system_font_height = height;
     }
-  glist = gtk_container_children (GTK_CONTAINER (notebook));
-  for (i = g_list_length (glist) - 1; i >= 0; i--)
+  for (i = misc_mdi_get_n_pages (MISC_MDI (mdi)) - 1; i >= 0; i--)
     {
-      vmaid = g_object_get_data (G_OBJECT(g_list_nth_data(glist,i)),
-                                                                "user_data");
+      VmaidWindow *vmaid;
+
+      vmaid = misc_mdi_get_data (MISC_MDI (mdi), i);
       gtk_widget_set_usize (vmaid->drawing, vmaid->width * 3 + 4,
                         system_font_height * 2 + ((vmaid->avi_edit[0] ? 1 : 0)
                             + (vmaid->avi_edit[1] ? 1 : 0)) * vmaid->height);
       gtk_widget_queue_resize (GTK_WIDGET (window));
       gtk_widget_draw (vmaid->drawing, NULL);
     }
-  g_list_free (glist);
 }
 
 
@@ -81,19 +80,8 @@ signal_style_set (GtkWidget *widget,
 *                                                                             *
 ******************************************************************************/
 void
-signal_activate_menu_window (GtkWidget *widget,
-                             GtkWidget *child)
-{
-  gtk_notebook_set_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_page_num (GTK_NOTEBOOK (notebook), child));
-  gtk_widget_grab_focus (((VmaidWindow *)
-                g_object_get_data (G_OBJECT (child), "user_data"))->drawing);
-}
-
-
-void
-signal_activate_menu_history (GtkWidget *widget,
-                              gchar     *file)
+signal_activate_menu_history (GtkWidget   *widget,
+                              const gchar *file)
 {
   FileOpen *file_open;
 
@@ -108,48 +96,34 @@ signal_activate_menu_history (GtkWidget *widget,
 }
 
 
-void
-signal_destroy_menu_history (GtkWidget *widget,
-                             gchar     *file)
-{
-  g_free (file);
-}
-
-
 /******************************************************************************
 *                                                                             *
-* ja:シグナル/イベント関数群(ノート)                                          *
+* ja:シグナル/イベント関数群(MDI)                                             *
 *                                                                             *
 ******************************************************************************/
 void
-signal_clicked (GtkWidget *widget,
-                GtkWidget *child)
+signal_close_window (GtkWidget   *widget,
+                     guint        page_num,
+                     VmaidWindow *vmaid)
 {
-  if (prompt_close ((VmaidWindow *)g_object_get_data (G_OBJECT (child),
-                                                                "user_data")))
-    gtk_notebook_remove_page(GTK_NOTEBOOK(notebook),
-                        gtk_notebook_page_num(GTK_NOTEBOOK(notebook),child));
+  if (prompt_close (vmaid))
+    gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page_num);
 }
 
 
 void
-signal_switch_page (GtkNotebook     *notebook,
-                    GtkNotebookPage *page,
-                    gint             page_num,
-                    gpointer         user_data)
+signal_switch_window (GtkWidget   *widget,
+                      guint        page_num,
+                      VmaidWindow *vmaid)
 {
-  VmaidWindow *vmaid;
-
-  vmaid = g_object_get_data
-        (G_OBJECT (gtk_notebook_get_nth_page(notebook,page_num)), "user_data");
   /* ja:メニュー表示 */
-  set_menu_bar(vmaid);
+  set_menu_bar (vmaid);
 }
 
 
 void
-signal_destroy_notebook (GtkWidget *widget,
-                         gpointer   user_data)
+signal_destroy_mdi (GtkWidget *widget,
+                    gpointer   user_data)
 {
   gtk_timeout_remove (timer_id);
 }
@@ -281,19 +255,17 @@ signal_selection_received (GtkWidget        *widget,
       else if (data->type == atom_scenario)
         {
           gboolean result = TRUE, paste[2];
-          gint max, sx;
+          gint max, sx, page_num;
           AviEdit **avi_edit;
           VmaidWindow *vmaid;
           VmaidHistory *d;
 
           /* ja:貼り付け */
-          avi_edit = avi_from_scenario(data->data);
+          avi_edit = avi_from_scenario (data->data);
           if (!avi_edit)
             return;
-          vmaid = g_object_get_data (G_OBJECT
-                    (gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook),
-                    gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook)))),
-                                                                "user_data");
+          page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi));
+          vmaid = misc_mdi_get_data (MISC_MDI (mdi), page_num);
           if (vmaid->select.stream < 0)
             {
               /* ja:選択範囲がないときUNDOの準備 */
@@ -434,7 +406,7 @@ signal_selection_received (GtkWidget        *widget,
           misc_set_scroll_bar (vmaid->hscroll,
                                 GTK_SIGNAL_FUNC (signal_value_changed_hscroll),
                                                 vmaid, 0, max, sx, vmaid->top);
-          vmaid_change_edit (vmaid, TRUE);
+          misc_mdi_set_edited (MISC_MDI (mdi), page_num, TRUE);
         }
     }
 }
@@ -459,8 +431,8 @@ signal_selection_recv (GObject   *object,
 
   /* ja:プロセス間通信 */
   /* ja:ファイルの履歴 */
-  arg_h.val = history;
-  arg_h.def = 4;
+  arg_h.val = misc_history_get_num (MISC_HISTORY (history));
+  arg_h.def = MISC_HISTORY_NUM_DEF;
   /* ja:新規ファイル */
   arg_n.flag = newfile;
   arg_n.def = FALSE;
@@ -477,21 +449,7 @@ signal_selection_recv (GObject   *object,
   arg_x.def = FALSE;
   /* ja:コマンドラインの解析 */
   arg_files = arg_analyse (argv, arg_opts);
-  if (history != arg_h.val && 0 <= arg_h.val && arg_h.val <= 10)
-    {
-      /* ja:履歴の整理 */
-      GList *glist;
-      GtkWidget *menu_shell;
-
-      history = arg_h.val;
-      menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/File");
-      glist = gtk_container_children (GTK_CONTAINER (menu_shell));
-      for (i = g_list_length (glist) - 3;
-                            i > history - (history == 0) + MENUFILE - 2; i--)
-        gtk_container_remove (GTK_CONTAINER
-                                    (menu_shell), g_list_nth_data (glist, i));
-      g_list_free (glist);
-    }
+  misc_history_set_num (MISC_HISTORY (history), arg_h.val);
   newfile = arg_n.flag;
   def_width = array[0];
   def_height = array[1];
@@ -525,15 +483,13 @@ signal_delete (GtkWidget *widget,
                gpointer   user_data)
 {
   gint page;
-  VmaidWindow *vmaid;
 
-  while ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook))) >= 0)
+  while ((page = gtk_notebook_get_current_page (GTK_NOTEBOOK (mdi))) >= 0)
     {
-      vmaid = g_object_get_data (G_OBJECT (gtk_notebook_get_nth_page
-                                (GTK_NOTEBOOK (notebook), page)), "user_data");
-      if (!prompt_close (vmaid))
+      if (!prompt_close ((VmaidWindow *)
+                                    misc_mdi_get_data (MISC_MDI (mdi), page)))
         return TRUE;
-      gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page);
+      gtk_notebook_remove_page (GTK_NOTEBOOK (mdi), page);
     }
 #ifdef USE_THREAD
   g_thread_create ((GThreadFunc)close_program, NULL, TRUE, NULL);
@@ -542,22 +498,3 @@ signal_delete (GtkWidget *widget,
 #endif /* not USE_THREAD */
   return TRUE;
 }
-
-
-void
-signal_destroy_window (GtkWidget *widget,
-                       gpointer   user_data)
-{
-  gint i;
-  GList *glist;
-  GtkWidget *menu_shell;
-
-  g_memset (history_files, 0, sizeof (gchar *) * 10);
-  menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/File");
-  glist = gtk_container_children (GTK_CONTAINER (menu_shell));
-  for (i = 0; i < (gint)g_list_length (glist) - MENUFILE - 1; i++)
-    history_files[i] = g_strdup (g_object_get_data (G_OBJECT
-                    (g_list_nth_data (glist, i + MENUFILE - 1)), "user_data"));
-  g_list_free (glist);
-  gtk_main_quit ();
-}
