@@ -1,6 +1,6 @@
 /*
     Video maid
-    copyright (c) 1998-2004 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2005 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -32,6 +32,9 @@
 #include "misc/license.h"
 #include "misc/misc_accel.h"
 #include "misc/misc_comm.h"
+#include "misc/misc_history.h"
+#include "misc/misc_mdi.h"
+#include "misc/misc_winmenu.h"
 #include "misc/profile.h"
 
 
@@ -40,21 +43,20 @@ gboolean newfile = FALSE;           /* ja:TRUE:新規ファイル,FALSE:なし *
 gboolean second = FALSE;            /* ja:TRUE:新規,FALSE:既存 */
 gboolean default_ratio = FALSE;     /* ja:TRUE:縦/横の比率を保持しない */
 gboolean file_merge = FALSE;        /* ja:TRUE:連番,FALSE:単独 */
-gchar *history_files[10];           /* ja:ファイルの履歴 */
 gchar *open_path = NULL;            /* ja:開くのパス */
 gchar *save_path = NULL;            /* ja:名前を付けて保存のパス */
 gchar *clipboard_scenario = NULL;   /* ja:クリップボードのテキスト */
 gint def_width, def_height;         /* ja:デフォルトのウインドウのサイズ */
-gint history = 4;                   /* ja:ファイルの履歴 */
 gint default_view = 6;              /* ja:ウインドウが作成時の初期値 */
 guint default_rate = 15;            /* ja:ウインドウが作成時の初期値 */
 guint timer_id = 0;                 /* ja:タイマID */
+GObject *history;                   /* ja:ファイルの履歴 */
 GdkAtom atom_targets;               /* ja:ターゲット */
 GdkAtom atom_scenario;              /* ja:シナリオオブジェクト */
 GdkBitmap *bitmap0;                 /* ja:波背景 */
 GdkBitmap *bitmap1;                 /* ja:網背景 */
 GtkWidget *window;                  /* ja:ウインドウ */
-GtkWidget *notebook;                /* ja:ノートブック */
+GtkWidget *mdi;                     /* en:MDI */
 GtkWidget *tview;                   /* ja:リストボックス */
 GtkWidget *status;                  /* ja:ステータスウインドウ */
 GtkPositionType n_pos = GTK_POS_TOP;/* ja:ノートブックの位置 */
@@ -96,16 +98,14 @@ main (int   argc,
   gint i, count = 0, ascent, height;
   FileOpen *file_open;
   Profile *profile;
-  GObject *accel;
-  GdkBitmap *mask;
+  GObject *accel, *winmenu;
   GdkPixmap *pixmap;
   GtkAccelGroup *accel_group;
   GtkCellRenderer *renderer;
   GtkStyle *style;
   GtkTreeSelection *select;
   GtkTreeViewColumn *column;
-  GtkWidget *toolbar, *text, *handle, *scroll, *vbox;
-  GtkWidget *menu_bar, *menu_shell, *menu_item;
+  GtkWidget *menu_bar, *toolbar, *text, *handle, *scroll, *vbox;
   gchar *title[4] = {N_("Mode"), N_("File"), N_("Time"), N_("Status")};
   GtkItemFactoryEntry menu_items[] = {
 {N_("/_File"),                    NULL,          NULL,             0, "<Branch>"},
@@ -207,18 +207,23 @@ main (int   argc,
 
   def_width = gdk_screen_width () * 2 / 3;
   def_height = gdk_screen_height() * 2 / 3;
-  g_memset (history_files, 0, sizeof (gchar *) * 10);
+
+  /* ja:ファイルの履歴 */
+  history = misc_history_new ();
+
   /* ja:レジストリから読み出す */
   profile = profile_open (NULL);
   if (profile)
     {
+      gint num;
+
       if (!profile_get_value (profile, "vmaid", "license",
                 (gpointer)&license, sizeof (gboolean),
                                                     PROFILE_VALUE_TYPE_BOOL))
         license = FALSE;
-      if (!profile_get_value (profile, "vmaid", "history",
-                (gpointer)&history, sizeof (gint), PROFILE_VALUE_TYPE_INT))
-        history = 4;
+      if (profile_get_value (profile, "vmaid", "history",
+            (gpointer)&num, sizeof (gint), PROFILE_VALUE_TYPE_INT))
+        misc_history_set_num (MISC_HISTORY (history), num);
       if (!profile_get_value (profile, "vmaid", "newfile",
                 (gpointer)&newfile, sizeof (gboolean),
                                                     PROFILE_VALUE_TYPE_BOOL))
@@ -260,21 +265,19 @@ main (int   argc,
                 ? g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL) : NULL;
       g_free (utf8str);
       /* ja:ファイルの履歴 */
-      for (i = 0; i < history; i++)
+      for (i = misc_history_get_num (MISC_HISTORY (history)) - 1; i >= 0; i--)
         {
           key = g_strdup_printf ("history%d", i);
           utf8str = profile_get_string (profile, "vmaid", key);
-          history_files[i] = utf8str
-                ? g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL) : NULL;
-          g_free (utf8str);
           g_free (key);
-          if (!history_files[i])
-            break;
-          if (*history_files[i] == '\0')
+          if (utf8str)
             {
-              g_free (history_files[i]);
-              history_files[i] = NULL;
-              break;
+              gchar *file;
+
+              file = g_filename_from_utf8 (utf8str, -1, NULL, NULL, NULL);
+              g_free (utf8str);
+              misc_history_add_file (MISC_HISTORY (history), file);
+              g_free (file);
             }
         }
       profile_close (profile);
@@ -310,8 +313,8 @@ main (int   argc,
 {NULL,      '\0', NULL,           NULL,              0,                   NULL}};
 
     /* ja:ファイルの履歴 */
-    arg_h.val = history;
-    arg_h.def = 4;
+    arg_h.val = misc_history_get_num (MISC_HISTORY (history));
+    arg_h.def = MISC_HISTORY_NUM_DEF;
     /* ja:新規ファイル */
     arg_n.flag = newfile;
     arg_n.def = FALSE;
@@ -335,8 +338,7 @@ VMAID_COPYRIGHT_NOTICE"\n"
     /* ja:ヘルプ */
     arg_q.str = VMAID_COPYRIGHT_NOTICE"\n\n";
     arg_files = arg_analyse (argv, arg_opts);
-    if (0 <= arg_h.val && arg_h.val <= 10)
-      history = arg_h.val;
+    misc_history_set_num (MISC_HISTORY (history), arg_h.val);
     def_width = array[0];
     def_height = array[1];
     second = arg_x.flag;
@@ -349,7 +351,7 @@ VMAID_COPYRIGHT_NOTICE"\n"
   g_signal_connect (G_OBJECT (window), "delete-event",
                                             G_CALLBACK (signal_delete), NULL);
   g_signal_connect (G_OBJECT (window), "destroy",
-                                    G_CALLBACK (signal_destroy_window), NULL);
+                                            G_CALLBACK (gtk_main_quit), NULL);
   gtk_widget_realize (window);
   /* en:Drag and Drop */
   gtk_drag_dest_set (window, GTK_DEST_DEFAULT_MOTION
@@ -372,14 +374,16 @@ VMAID_COPYRIGHT_NOTICE"\n"
                                 G_CALLBACK (signal_selection_get), NULL);
   g_signal_connect (G_OBJECT (window), "selection-received",
                                 G_CALLBACK (signal_selection_received), NULL);
-  /* ja:ノートブック */
-  notebook = gtk_notebook_new ();
-  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
-  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook), n_pos);
-  g_signal_connect (G_OBJECT (notebook), "switch-page",
-                                G_CALLBACK (signal_switch_page), NULL);
-  g_signal_connect (GTK_OBJECT (notebook), "destroy",
-                                G_CALLBACK (signal_destroy_notebook), NULL);
+  /* en:MDI */
+  mdi = misc_mdi_new ();
+  gtk_notebook_set_scrollable (GTK_NOTEBOOK (mdi), TRUE);
+  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (mdi), n_pos);
+  g_signal_connect (G_OBJECT (mdi), "close-window",
+                                G_CALLBACK (signal_close_window), NULL);
+  g_signal_connect (G_OBJECT (mdi), "switch-window",
+                                G_CALLBACK (signal_switch_window), NULL);
+  g_signal_connect (GTK_OBJECT (mdi), "destroy",
+                                G_CALLBACK (signal_destroy_mdi), NULL);
   /* ja:ポップアップ */
   accel_group = gtk_accel_group_new ();
   ifactory_popup = gtk_item_factory_new (GTK_TYPE_MENU, "<main>", accel_group);
@@ -406,6 +410,16 @@ VMAID_COPYRIGHT_NOTICE"\n"
                         sizeof (menu_items) / sizeof (GtkItemFactoryEntry),
                                                             menu_items, NULL);
   menu_bar = gtk_item_factory_get_widget (ifactory_menu, "<main>");
+  /* ja:ファイルの履歴 */
+  misc_history_set_menu (MISC_HISTORY (history),
+    gtk_item_factory_get_widget (ifactory_menu, "<main>/File"), MENUFILE - 2);
+  g_signal_connect (MISC_HISTORY (history), "activate",
+                            G_CALLBACK (signal_activate_menu_history), NULL);
+  /* ja:ウインドウメニュー */
+  winmenu = misc_winmenu_new ();
+  misc_winmenu_set_mdi (MISC_WINMENU (winmenu), MISC_MDI (mdi));
+  misc_winmenu_set_menu (MISC_WINMENU (winmenu),
+            gtk_item_factory_get_widget (ifactory_menu, "<main>/Window"), 0);
   /* ja:アクセルグループ */
   accel = misc_accel_new ();
   misc_accel_set_window (MISC_ACCEL (accel), GTK_WINDOW (window));
@@ -422,14 +436,16 @@ VMAID_COPYRIGHT_NOTICE"\n"
       }
     else
       {
+        GdkBitmap *mask;
+
         pixmap = gdk_pixmap_create_from_xpm_d (window->window,
                                             &mask, NULL, toolbar_items[i].xpm);
         toolbar_items[i].item = gtk_toolbar_append_item
-                        (GTK_TOOLBAR(toolbar),
-                        NULL, _(toolbar_items[i].name), NULL,
-                        gtk_pixmap_new (pixmap,mask),
-                        G_CALLBACK (toolbar_items[i].callback),
-                        toolbar_items[i].user_data);
+                                (GTK_TOOLBAR (toolbar),
+                                NULL, _(toolbar_items[i].name), NULL,
+                                gtk_pixmap_new (pixmap, mask),
+                                G_CALLBACK (toolbar_items[i].callback),
+                                toolbar_items[i].user_data);
       }
   /* ja:リストボックス */
   list_store = gtk_list_store_new (5, G_TYPE_STRING,
@@ -474,30 +490,13 @@ VMAID_COPYRIGHT_NOTICE"\n"
   gtk_container_set_border_width (GTK_CONTAINER (handle), 2);
   gtk_container_add (GTK_CONTAINER (handle), toolbar);
   gtk_box_pack_start (GTK_BOX (vbox), handle, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox), notebook, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), mdi, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vbox), scroll, TRUE, TRUE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), status, FALSE, FALSE, 0);
   gtk_box_pack_end (GTK_BOX (vbox), text, FALSE, FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), vbox);
-  /* ja:ファイルの履歴 */
-  menu_shell = gtk_item_factory_get_widget (ifactory_menu, "<main>/File");
-  for (i = 0; i < history && history_files[i]; i++)
-    {
-      utf8str = g_filename_to_utf8 (history_files[i], -1, NULL, NULL, NULL);
-      menu_item = gtk_menu_item_new_with_label (utf8str);
-      g_free (utf8str);
-      g_signal_connect (G_OBJECT (menu_item), "activate",
-                G_CALLBACK (signal_activate_menu_history), history_files[i]);
-      g_signal_connect (G_OBJECT (menu_item), "destroy",
-                G_CALLBACK (signal_destroy_menu_history), history_files[i]);
-      g_object_set_data (G_OBJECT (menu_item), "user_data", history_files[i]);
-      gtk_menu_insert (GTK_MENU (menu_shell), menu_item, i + MENUFILE - 1);
-    }
-  if (i>0)
-    gtk_menu_insert (GTK_MENU (menu_shell), gtk_menu_item_new (),
-                                                            i + MENUFILE - 1);
   /* ja:タイマ */
-  timer_id = gtk_timeout_add (500, signal_timeout, NULL);
+  timer_id = g_timeout_add (500, signal_timeout, NULL);
 
   /* ja:メニュー表示 */
   set_menu_bar (NULL);
@@ -586,10 +585,14 @@ VMAID_COPYRIGHT_NOTICE"\n"
   profile = profile_open (NULL);
   if (profile)
     {
+      gint num;
+      GList *glist;
+
+      num = misc_history_get_num (MISC_HISTORY (history));
       profile_set_value (profile, "vmaid", "license",
             (gpointer)&license, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
       profile_set_value (profile, "vmaid", "history",
-            (gpointer)&history, sizeof (gint), PROFILE_VALUE_TYPE_INT);
+            (gpointer)&num, sizeof (gint), PROFILE_VALUE_TYPE_INT);
       profile_set_value (profile, "vmaid", "newfile",
             (gpointer)&newfile, sizeof (gboolean), PROFILE_VALUE_TYPE_BOOL);
       profile_set_value (profile, "vmaid", "second",
@@ -634,10 +637,11 @@ VMAID_COPYRIGHT_NOTICE"\n"
             }
         }
       /* ja:ファイルの履歴 */
-      for (i = 0; i < history && history_files[i]; i++)
+      glist = misc_history_get_file (MISC_HISTORY (history));
+      for (i = 0; i < g_list_length (glist); i++)
         {
           key = g_strdup_printf ("history%d", i);
-          utf8str = g_filename_to_utf8 (history_files[i], -1,
+          utf8str = g_filename_to_utf8 (g_list_nth_data (glist, i), -1,
                                                             NULL, NULL, NULL);
           if (utf8str)
             {
@@ -648,7 +652,8 @@ VMAID_COPYRIGHT_NOTICE"\n"
             }
           g_free (key);
         }
-      while (i < 10)
+      g_list_free (glist);
+      while (i <= MISC_HISTORY_NUM_MAX)
         {
           key = g_strdup_printf ("history%d", i++);
           profile_delete_key (profile, "vmaid", key);
@@ -660,8 +665,6 @@ VMAID_COPYRIGHT_NOTICE"\n"
   g_free (clipboard_scenario);
   g_free (open_path);
   g_free (save_path);
-  for (i = 0; i < history; i++)
-    g_free (history_files[i]);
   gtk_exit (0);
   return 0;
 }
